#include "helpers.h"
#include <stdio.h>
#include <stdlib.h>

void regex_error(const regex_t* regex, int status) {
    if(status > REG_NOMATCH) {
        char error_msg[100];
        regerror(status, regex, error_msg, sizeof(error_msg));
        fprintf(stderr, "Regex compilation failed: %s\n", error_msg);
        exit(EXIT_FAILURE);
    }
}

regex_t regex(const char* pattern) {
    regex_t regex;
    regex_error(&regex, regcomp(&regex, pattern, REG_EXTENDED));

    return regex;
}

bool regex_search(regmatch_t* match, const regex_t* regex, const char* string) {
    int status = regexec(regex, string, 1, match, 0);
    regex_error(regex, status);

    return !status;
}

const char* read_file(const char* filename) {
    FILE* file = fopen(filename, "r");
    if(file == NULL) {
        perror("Failed to open file");
        exit(EXIT_FAILURE);
    }

    fseek(file, 0, SEEK_END);
    size_t size = ftell(file);
    rewind(file);

    char* buffer = malloc(size + 1);
    fread(buffer, size, 1, file);
    fclose(file);

    return buffer;
}
