#include "codegen.h"
#include "compiler.h"
#include "helpers.h"
#include "lexer.h"
#include "node.h"
#include "parser.h"
#include "token.h"
#include <assert.h>
#include <stdbool.h>
#include <stdio.h>

int main(int argc, char** argv) {
    assert(argc > 1);
    const char* text = read_file(argv[1]);

    TokenRule rules[] = {
        { regex("^[A-Za-z][A-Za-z0-9]*"), TOKEN_IDENTIFIER },
        { regex("^[ \r\n]+"), TOKEN_WHITESPACE },
        { regex("^\".*?\""), TOKEN_STRING },
        { regex("^[0-9]+"), TOKEN_NUMBER },
        { regex("^;"), TOKEN_SEMICOLON },
        { regex("^\\("), TOKEN_OPAREN },
        { regex("^\\{"), TOKEN_OBRACE },
        { regex("^\\)"), TOKEN_CPAREN },
        { regex("^\\}"), TOKEN_CBRACE },
    };

    Lexer lexer = lexer_create(rules, c_arraylen(rules));
    lexer_feed(&lexer, text);

    Compiler compiler = {
        .lexer = &lexer,
        .types = c_make(cmap_str,
            {
                { "i8", "b" },
                { "i16", "h" },
                { "i32", "w" },
                { "i64", "l" },
                { "void", "" },
            }),
    };

    Token token;
    while(next(&compiler, &token)) {
        // printf("%c: %s\n", token.type, token.value);
        Node node = parse_token(&compiler, token);
        codegen(node);
    }

    return 0;
}
